#include <QPainterPath>

#include <string>

static void workaroundAddPolygon(QPainterPath& destPath, const QPolygonF& polygon) {
    bool isFirstPoint = true;
    for (const auto point : polygon) {
        if (isFirstPoint) {
            isFirstPoint = false;
            destPath.moveTo(point);
        } else {
            destPath.lineTo(point);
        }
    }
}

static void builtinAddPolygon(QPainterPath& destPath, const QPolygonF& polygon) {
	destPath.addPolygon(polygon);
}

enum class Mode { Invalid, BuiltIn, Workaround };

Mode parseMode(const char* arg) {
	const std::string modeString = arg;

	Mode mode = Mode::Invalid;
	if (modeString == "builtin") {
		mode = Mode::BuiltIn;
	} else if (modeString == "workaround") {
		mode = Mode::Workaround;
	}

	return mode;
}

int main(int argc, char* argv[]) {
	Q_ASSERT(argc == 3);

	const auto mode = parseMode(argv[1]);
	if (mode == Mode::Invalid) {
		throw std::runtime_error("invalid mode");
	}
	const int iterations = std::stoi(argv[2]);

	const QRectF rect { 1, 2, 3, 4 };
	QPainterPath pp;
	const auto addPolygon = (mode == Mode::BuiltIn ? builtinAddPolygon : workaroundAddPolygon);

	for (int i = 0; i < iterations; ++i) {
		addPolygon(pp, rect);
	}

	return 0;
}
